set gain 20.0
set mix 128.0
set vpitch 48.0
set reso1 126.0
set freq1 90.0
set zrezo1 -1.0
set zfreq1 0.0
set vibrato 30.0

proc myExit {} {
    global freq1
    puts stdout [format "NoteOff          0.0 1 %f 127" $freq1 ]
    puts stdout [format "ExitProgram"]
    flush stdout
    exit
}

proc noteOn {pitchVal pressVal} {
    global gain 
    set gain 32.0
    puts stdout [format "NoteOn           0.0 1 %f %f" $pitchVal $pressVal]
    flush stdout
}

proc noteOff {pitchVal pressVal} {
    global gain
    set gain 0.0
    puts stdout [format "NoteOff          0.0 1 %f %f" $pitchVal $pressVal]
    flush stdout
}

proc eqGainZeroes {} {
    puts stdout [format "EqualGainZeroes  0.0 1"]
    flush stdout
}

proc wackoZeroes {} {
    global zrezo1
    global zfreq1
    set zrezo1 -1.0
    set zfreq1 -1.0
    eqGainZeroes
}

proc printWhatz {tag value1 value2 } {
    puts stdout [format "%s %i %f" $tag $value1 $value2]
    flush stdout
}

proc changeGain {value} {
    global gain
    set gain $value
    flush stdout
}

proc changePitch {value} {
    global freq1
    set freq1 $value     
    puts stdout [format "PitchBend        0.0 1 %f" $freq1]
    flush stdout
}

proc changeVPitch {value} {
    global vpitch
    set vpitch $value     
    flush stdout
}

. config -bg grey20

frame .left -bg grey20
frame .right -bg grey20

frame .reso -bg black

button .reso.on -text NoteOn -bg grey66 -command { noteOn $freq1 $gain }
button .reso.off -text NoteOff -bg grey66 -command { noteOff $freq1 127.0 }
button .reso.eqGainZeroes -text EqGainZeroes -bg grey66 -command { wackoZeroes  }
button .reso.exit -text "Exit Program" -bg grey66 -command myExit

scale .left.gain -from 0 -to 128 -length 256 \
-variable gain -command {printWhatz "AfterTouch       0.0" 1} \
-orient horizontal -label "Gain" \
-tickinterval 32 -showvalue true -bg grey66

scale .left.mix -from 0 -to 128 -length 256 \
-variable mix -command {printWhatz "Mix       0.0" 1} \
-orient horizontal -label "<-------Voiced / Noise ------->" \
-tickinterval 32 -showvalue true -bg grey66 

scale .left.vpitch -from 0 -to 128 -length 256 \
-command {printWhatz "VPitch    0.0" 1} \
-variable vpitch \
-orient horizontal -label "MIDI Note Number Voice" \
-tickinterval 32 -showvalue true -bg grey66

scale .left.vibrato -from 0 -to 128 -length 256 \
-command {printWhatz "Vibrato    0.0" 1} \
-variable vibrato \
-orient horizontal -label "(6 hz.) Vibrato Amount" \
-tickinterval 32 -showvalue true -bg grey66

scale .right.reso1 -from 0 -to 128 -length 256 \
-command {printWhatz "Reso1    0.0 " 1} \
-orient horizontal -label "Resonance (*128)" \
-tickinterval 32 -showvalue true -bg grey66\
-variable reso1

scale .right.freq1 -from 0 -to 137 -length 256 \
-command {changePitch }  -variable freq1 \
-orient horizontal -label "MIDI Note Number for Resonator" \
-tickinterval 50 -showvalue true -bg grey66

scale .right.zreso1 -from -1 -to 150 -length 256 \
-command {printWhatz "ZReso1    0.0" 1} \
-orient horizontal -label "Zero Anti-Resonance (*128)" \
-tickinterval 50 -showvalue true -bg grey66\
-variable zrezo1

scale .right.zfreq1 -from -1 -to 137 -length 256 \
-command {printWhatz "ZPitch1    0.0" 1} \
-orient horizontal -label "Zero Anti-Resonance Freq. (MIDI#)" \
-tickinterval 50 -showvalue true -bg grey66\
-variable zfreq1

pack .reso.on -side left -padx 5
pack .reso.off -side left -padx 5
pack .reso.eqGainZeroes -side left -padx 5
pack .reso.exit -side left -padx 5
pack .reso -pady 10
pack .left.gain 	-padx 10 -pady 5
pack .left.mix 		-padx 10 -pady 5
pack .left.vpitch	-padx 10 -pady 5
pack .left.vibrato	-padx 10 -pady 5
pack .left -side left
pack .right.reso1 	-padx 10 -pady 5
pack .right.freq1 	-padx 10 -pady 5
pack .right.zreso1 	-padx 10 -pady 5
pack .right.zfreq1 	-padx 10 -pady 5
pack .right -side left
bind all <KeyPress> {
    noteOn $freq1 $gain 
}

wackoZeroes
eqGainZeroes
eqGainZeroes
eqGainZeroes

